#!/usr/bin/perl -w

# $Id: spamreport.pl,v 1.8 2003/03/31 02:50:00 dredd Exp $

use strict;
use Mail::Sendmail;

my $send_as = 'dev-null@megacity.org';
my $bcc = '';

my @mons = ('Jan','Feb','Mar','Apr','May','Jun',
            'Jul','Aug','Sep','Oct','Nov','Dec');

my @dont_bother = (
		   'eatme@metallicasucks.com',
		   'chrismas@megacity.org',
		   'mstubbe@megacity.org',
		   'greglivo@alphacon.org',
		   'tom@megacity.org',
		   'buckaroo.bonzai@hongkongcavaliers.com',
		   '@rfc-ignorant.org'
		   );

my @only_our_domains;

open CW, "< /etc/mail/local-host-names";
while (<CW>)
{
    chomp;
    push @only_our_domains, ($_);
}
close CW;

my ($d,$m) = (localtime(time-86400))[3,4];
my $date_str = sprintf "$mons[$m] %2s", $d;

my %reports = ();
my %same_as = (
	       'rderoo@' => 'rderoo@deroo.net',
               'resume@deroo.net' => 'rderoo@deroo.net',

               'jderoo@' => 'jderoo@deroo.net',

               'dredd' => 'dredd@megacity.org',
	       'whois@megacity.org' => 'dredd@megacity.org',
               'derek@' => 'dredd@megacity.org',
	       'webmaster@metallicasucks.com' => 'dredd@megacity.org',

	       'bc@' => 'bc@bjb.org',
               'dkh@' => 'dkh@bjb.org',
	       '@bjb.org' => 'bc@bjb.org',

	       '@drakkar.org' => 'dave@drakkar.org',

               );


process_file('/var/log/spamlog.0');
process_file('/var/log/spamlog');

foreach my $address (keys %reports)
{
    my $dont_bother_flag = 0;
    foreach my $db_regex (@dont_bother)
    {
	$dont_bother_flag = 1 if $address =~ /$db_regex/;
    }
    next if $dont_bother_flag;

    my $we_care = 0;
    foreach my $care_domain (@only_our_domains)
    {
	$we_care = 1 if $address =~ /\@$care_domain/i;
    }
    next unless $we_care;
    my $footer = <<EOF

If you are receiving this, it is because your mail is handled by
mail.megacity.org, which has VERY strict, some might even say anal,
spam-filtering rules. This report is generated automatically every
morning at midnight GMT, and is only sent if you had mail that was
blocked by our system.
EOF
;

    my $report = "";
    foreach my $sender (keys %{$reports{$address}})
    {
      foreach my $orig_recip (keys %{$reports{$address}{$sender}})
      {
	foreach my $iphost (keys %{$reports{$address}{$sender}{$orig_recip}})
	{
	  foreach my $type (keys %{$reports{$address}{$sender}{$orig_recip}{$iphost}})
	  {
	    foreach my $fmt_msg (keys %{$reports{$address}{$sender}{$orig_recip}{$iphost}{$type}})
	    {
	      my @counts = @{$reports{$address}{$sender}{$orig_recip}{$iphost}{$type}{$fmt_msg}};
	      my $count = $counts[0];
	      my $report_string = sprintf "SENDER:          %-30s\nRECIPIENT ADDR:  %-30s\nCONNECTION INFO: %-30s\n", $sender, $orig_recip, $iphost;
	      $report_string .= sprintf "RULESET:         %-30s\n", $type;
	      if ($count == 1)
	      {
		$report_string .= sprintf "TIME:            %s\n", $counts[1];
	      }
	      else
	      {
		$report_string .= sprintf "ATTEMPTS:        %d ( %s", $count, $counts[1];
		if ($count == 2)
		{
		  $report_string .= sprintf ", %s", $counts[2];
		}
		else
		{
		  if ($count <= 3)
		  {
		    $report_string .= sprintf ", %s, %s", $counts[2], $counts[3];
		  }
		  else
		  {
		    $report_string .= sprintf " - %s", $counts[3];
		  }
		}
		$report_string .= sprintf " )\n";
	      }
	      $report_string .= sprintf "DETAIL:          $fmt_msg\n\n";
	      $report .= $report_string;
	    }
	  }
	}
      }
    }
    $report .= "\n" . $footer;
    my %mail_msg = (
		    To => $address,
		    Bcc => $bcc,
		    From => $send_as,
		    Subject => "Spamblocking report for $address / $date_str",
		    Message => $report
		    );
   sendmail(%mail_msg);
}


sub process_file
{

# sample line
# Dec 15 03:16:00 minbar spamcheck[28203]: 211.173.193.120/[211.173.193.120] 551 <j24861@thai2k.com> <bderoo@wakepro.com> connect_ip_dnsbl "Message rejected because the IP address (211.173.193.120) is listed in the MAPS RBL DUL or RSS - www.mail-abuse.org"

  my $file = shift;
  open FILE, "< $file" or die "Couldn't open $file : $!\n";
  while (<FILE>)
  {
    chomp;
	next if ! /^$date_str/;
    next if /last message repeated \d times/;
    my $line = $_;
    my @stuff = (split /\s+/);
    my ($time,$iphost) = @stuff[2,5];
    my ($sender,$recip,$type,$rstring) = $line =~ /(\<.*?\>) (\<.*?\>) (\S+) (.*)/;
    my @rest = split /\s+/, $rstring;

    if (! defined $sender)
    {
      ($sender,$recip,$type,@rest) = @stuff[7..$#stuff];
    }

    my $orig_recip = $recip;
    $recip = lc $recip;
    $recip =~ s/^\<//; $recip =~ s/\>$//;
    foreach my $regex (keys %same_as)
    {
      if ($recip =~ /$regex/)
      {
        $recip = $same_as{$regex};
        last;
      }
    }

    my $message = join ' ', @rest;
    my $formatted_message = $message;
    if ((length $message) > 40)
    {
        my @lines = ();
        my $temp_line;
        foreach my $word (@rest)
        {
          if ( (defined $temp_line) and
	       ( (length "$temp_line $word") > 50) and
	       ($word =~ /\w/)
	     )
          {
	    push @lines, ($temp_line);
	    undef $temp_line;
	  }
	  if (defined $temp_line)
	  {
	    $temp_line .= " $word";
	  }
	  else
	  {
	    $temp_line = $word;
	  }
	}
	push @lines, ($temp_line) if defined $temp_line;
	$formatted_message = join "\n                 ", @lines;
      }

      my $conn_info = "$iphost \@ $time";
      if (not exists $reports{$recip})
      {
        $reports{$recip} = {$sender, {$orig_recip => {$iphost => {$type => {$formatted_message => [1, $time]}}}}};
      }
      else
      {
        if (not exists $reports{$recip}{$sender})
        {
          $reports{$recip}{$sender} = {$orig_recip => {$iphost => {$type => {$formatted_message => [1, $time]}}}};
        }
        else
	{
	  if (not exists $reports{$recip}{$sender}{$orig_recip})
	  {
	    $reports{$recip}{$sender}{$orig_recip} = {$iphost => {$type => {$formatted_message => [1, $time]}}};
	  }
	  else
	  {
	    if (not exists $reports{$recip}{$sender}{$orig_recip}{$iphost})
	    {
	      $reports{$recip}{$sender}{$orig_recip}{$iphost} = {$type => {$formatted_message => [1, $time]}};
	    }
	    else
	    {
	      if (not exists $reports{$recip}{$sender}{$orig_recip}{$iphost}{$type})
	      {
	        $reports{$recip}{$sender}{$orig_recip}{$iphost}{$type} = {$formatted_message => [1, $time]};
	      }
	      else
	      {
	        if (not exists $reports{$recip}{$sender}{$orig_recip}{$iphost}{$type}{$formatted_message})
	        {
	          $reports{$recip}{$sender}{$orig_recip}{$iphost}{$type} = {$formatted_message => [1, $time]};
	        }
	        else
	        {
		  my $counts = $reports{$recip}{$sender}{$orig_recip}{$iphost}{$type}{$formatted_message};
		  $$counts[0] = $$counts[0] + 1;
	          if ($$counts[0] < 3)
	          {
	            push @{$counts}, ( $time );
	          }
	          else
	          {
	            $$counts[3] = $time;
	          }
		}
	      }
	    }
	  }
	}
      }
    }
  }
